#include <UIPEthernet.h>
#include <EEPROM.h>

// DEBUG SERIAL OUT
// #define DEBUG 

// IP addres in my network
#define MACADDRESS 0xB0,0xE7,0x4C,0x53,0xF7,0x44 
#define MYIPADDR 192,168,234,175
// Default WinTest UDP Port
#define LISTENPORT 9871 

EthernetUDP udp;

// STATUS: "UKW" "" 0 18 1 0 34001100 "1" 0 "1"  34001000 ""�
#define NR_STAUS_ELEMENTS 13

// PINS für die QRG Schaltungen
#define PIN_1G3 PIN2
#define PIN_2G4 PIN3
#define PIN_3G4 PIN4
#define PIN_5G7 PIN5
#define PIN_10G PIN6
#define PIN_24G PIN7
#define PIN_47G  PIN_A0
#define PIN_76G  PIN_A1
#define PIN_134G PIN_A2
#define PIN_241G PIN_A3

byte pinsOut [] = { PIN_1G3, PIN_2G4, PIN_3G4, PIN_5G7, PIN_10G, PIN_24G, PIN_47G, PIN_76G, PIN_134G, PIN_241G};

struct baseConf
{
  uint8_t ip[4];
  long port;
  char station[15];
};

baseConf MyConf;


void select_band(byte pin) {
  if (digitalRead(pin) == LOW) {
    for (byte i = 0; i < sizeof(pinsOut); i++)
      digitalWrite(pinsOut[i], LOW);
    digitalWrite(pin, HIGH);  
  }
}

void setup() {

    Serial.begin(9600);
    Serial.println(F("DG1ATN WinTest Status Reader"));

    for (byte i = 0; i < sizeof(pinsOut); i++)
    {
      pinMode(pinsOut[i], OUTPUT);
      digitalWrite(pinsOut[i], LOW);
    }
    
    for (byte i = 0; i < sizeof(pinsOut); i++)
    {
      digitalWrite(pinsOut[i],HIGH);
      delay(500);
      digitalWrite(pinsOut[i],LOW);
    }

    #ifdef DEBUG
      Serial.println(F("read Config"));
    #endif

    EEPROM.get(0,MyConf);

    if (MyConf.ip[0] == 255)
    {
      uint8_t ip[4] = {MYIPADDR};
      for (byte i = 0; i < 4; i++)
        MyConf.ip[i] = ip[1];
    }

    if (MyConf.port == 255)
    {
      MyConf.port = LISTENPORT;
    }

    Serial.print(F("Station: "));
    Serial.println(MyConf.station);
    
    #ifdef DEBUG
      Serial.println(F("init network"));
    #endif

    uint8_t mac[6] = {MACADDRESS};

    Serial.print(F("Local IP: "));
    for (byte i = 0; i < 3; i++)
    {
      Serial.print( MyConf.ip[i]);
      Serial.print(F("."));
    }
    Serial.println(MyConf.ip[3]);

    Ethernet.begin(mac, MyConf.ip);

    Serial.print(F("WT UDP Port: "));
    Serial.println(MyConf.port);

    int success = udp.begin(MyConf.port);

    #ifdef DEBUG
      Serial.println(success ? "success":"failed");
    #endif
}

void loop() {

  int size = udp.parsePacket();
  if (size > 0) {
    #ifdef DEBUG
      Serial.println(size);
    #endif
    do
      {
        char* msg = (char*)malloc(size+1);
        int len = udp.read(msg,size+1);
        msg[len]=0;
        Serial.println(msg);
        String s(msg);
        free(msg);
        if (s.startsWith(F("STATUS:")))
        {
          bool stationOK;
          stationOK = false;
          for (byte i = 0; i < NR_STAUS_ELEMENTS; i++)
          {
            byte e = s.indexOf(F(" "));
            if (i == 1) 
            {
              String f = s.substring(0,e);
              String station = f.substring(1,f.length()-1);
              #ifdef DEBUG
                Serial.print(station);
                Serial.print(F("?"));
                Serial.println(MyConf.station);
              // Serial.println(strlen(MyConf.station));
              #endif
              if ((station == MyConf.station) or (strlen(MyConf.station) == 0))
              {
                stationOK = true;
                #ifdef DEBUG
  	                Serial.println("confirmed");
                #endif
              }
            }
            if ((i == 7) && (stationOK)) {
              String f = s.substring(0,e+1);
              #ifdef DEBUG
                Serial.println(f);
              #endif

              if (f.startsWith(F("1296")))
                select_band(PIN_1G3);
              if (f.startsWith(F("2320")))
                select_band(PIN_2G4);
              if (f.startsWith(F("34")))
                select_band(PIN_3G4);
              if (f.startsWith(F("57")))
                select_band(PIN_5G7);              
              if (f.startsWith(F("10")))
                select_band(PIN_10G);
              if (f.startsWith(F("24")))
                select_band(PIN_24G);
              if (f.startsWith(F("47")))
                select_band(PIN_47G);
              if (f.startsWith(F("76")))
                select_band(PIN_76G);
              if (f.startsWith(F("134")))
                select_band(PIN_134G);
              if (f.startsWith(F("241")))
                select_band(PIN_241G);
            }
            s.remove(0,e+1);
          }
        }
      }
    while ((size = udp.available())>0);
    //finish reading this packet:
    udp.flush();
  }

  udp.stop();
  //restart with new connection to receive packets from other clients
  int success = udp.begin(MyConf.port);
  if (not success)
    Serial.println(F("Open UDP Port failed!"));
  
  if (Serial.available() > 0)
  {
    String s ;
    s = Serial.readString();
    if (s.startsWith(F("IP:")))
    {
      Serial.print(F("IP:"));
      byte e = s.indexOf(F(":"));
      s.remove(0,e+1);
      for (byte i = 0;i < 4;i ++)
      {
        e = s.indexOf(F("."));
        if (e == -1)
          e = s.length();
        char c[5];
        s.substring(0,e+1).toCharArray(c,5);
        MyConf.ip[i] =  atoi(c);
        s.remove(0,e+1);
        if (i < 3){
          Serial.print(MyConf.ip[i]);
          Serial.print('.');
        }
        else
          Serial.println(MyConf.ip[i]);
      }
    }
    if (s.startsWith(F("Port:")))
    {
      byte e = s.indexOf(F(":"));
      s.remove(0,e+1);
      char c[15];
      s.toCharArray(c,s.length()+1);
      MyConf.port = atoi(c);
      Serial.print(F("Port:"));
      Serial.println(c);
    }
    if (s.startsWith(F("Station:")))
    {
      byte e = s.indexOf(F(":"));
      s.remove(0,e+1);
      s.toCharArray(MyConf.station,s.length()+1);
      Serial.print(F("Station:"));
      Serial.println(MyConf.station);
    }
    EEPROM.put(0,MyConf);
  }
}